c xvpl2el.f
c Convert positions and velocities in the PL.IN file to orbital elements.
c   - UPRAVENA VERZE PRO STELARNI APLIKACE!
c Miroslav Broz (miroslav.broz@email.cz), Jul 21st 2006

      program xvpl2el

c constants
      implicit none
      integer NPLMAX
      parameter(NPLMAX=10)

      real*8 pi,deg,rad,arcsec
      parameter(pi = 3.1415926535897932d0, deg = pi/180., rad = 180./pi,
     :  arcsec = deg/3600.)

      real*8 pc,day
      parameter(pc = 206264.8d0)
      parameter(day = 1.d0/365.25d0)

c input parameters
      integer npl, id
      real*8 t, y(6), elmts(6), m(0:NPLMAX), d

c temporary variables
      integer i, j
      real*8 a, e, i0, omega0, omega, m0
      real*8 varpi, P, n, t_Bessel, tau, tau_Bessel, a_angular,
     :  omega0_PA
      real*8 nula2pi, jdate2bessel

c write header
        write(*,30)
30      format("# t [JD] & id & a [AU] & e [] & i [deg] & Omega [deg]",
     :    " & omega [deg] & M [deg]     ",
     :    " & t [Besselian year] & a [arcsec] & P [yr] & P [day]",
     :    " & n [deg/yr] & varpi [deg] wrt. P.A. direction ",
     :    " & omega0 [deg] & tau [JD] & tau [Besselian year]")

c read distance and masses first
      read(*,*,end=990,err=990) d
      d = d*pc
      read(*,*,end=990,err=990) npl
      do i = 1, npl
        read(*,*,end=990,err=990) m(i)
      enddo

c read integration output
5     continue
        read(*,*,end=999,err=990) t, id, y(1), y(2), y(3),
     :    y(4), y(5), y(6)

c calculate elements
        call e1_new(y(1),y(4),elmts,m(1),m(id))

c and additional variables
        a      = elmts(1)
        e      = elmts(2)
        i0     = elmts(3)
        omega0 = elmts(4)  ! node
        omega  = elmts(5)  ! peri
        m0     = elmts(6)

c a**3/P**2 = m1+m2
c M = (t-tau)*n 

        a_angular = atan(a/d)
        omega0_PA = nula2pi(90.d0*deg - omega0)         ! the reference direction is UP in x, y plane! (as for the P.A.)
        varpi = nula2pi(90.d0*deg - (omega0+omega))
        P = sqrt(a**3.d0/(m(1)+m(id)))
        n = 2.d0*pi/P
        t_Bessel = jdate2bessel(t)
        tau = t - 365.25*m0/n
        tau_Bessel = jdate2bessel(tau)

c write output
        write(*,20) t, id, elmts(1), elmts(2),
     :    (nula2pi(elmts(j))*rad, j=3,6),
     :    t_Bessel, a_angular/arcsec, P, P/day, n*rad,
     :    varpi*rad, omega0_PA*rad,
     :    tau, tau_Bessel
20      format(f22.8,1x,i4,6(1x,f22.16),
     :    8x, f12.4, 6(1x,f20.12), 1x, f22.8, 1x, f12.4)

      goto 5

999   continue
      stop

990   continue
      write(*,*) '# Error reading standard input!'
      stop

      end

c**********************************************************************
      subroutine e1_new(x,v,elmts,m1,m2)
c**********************************************************************
c
c  determination of elemnts from the position and velocity
c  x(3)      rectagular coordinates of the body [au]
c  v(3)      velocity [au/day] 
c  elmts(6)  elements of orbit
c  m1       mass of the Sun (or zero)
c  m2       mass of the planet (or zero)
c
      implicit none
      double precision x(3),v(3),elmts(6),m1,m2
      double precision v2,r,a,xx,yy,zz,p,e,i0,e0,m0,v0,
     &  omega,omega0,sinv,cosv,nula2pi,gms,kg,gms2

c      parameter(gms=2.9591220663860403d-4,kg=0.017202098902128322400d0)
      parameter(gms=0.2959122082855911d-03)     ! a new value from JPLEPH

      gms2 = (m1+m2)*gms
      v2 = v(1)**2+v(2)**2+v(3)**2
      r = sqrt(x(1)**2+x(2)**2+x(3)**2)
      a = 1.d0/(2.d0/r-v2/gms2)
      xx = x(2)*v(3)-x(3)*v(2)
      yy = x(1)*v(3)-x(3)*v(1)
      zz = x(1)*v(2)-x(2)*v(1)
      p = (xx**2+yy**2+zz**2)/gms2
      e = sqrt(1.d0-p/a)
      i0 = atan(sqrt(xx**2+yy**2)/zz)
      omega = nula2pi(atan2(xx,yy))
      sinv = (x(1)*v(1)+x(2)*v(2)+x(3)*v(3))/r*sqrt(p/gms2)
      cosv = (p/r-1.d0)
      v0 = nula2pi(atan2(sinv,cosv))
      e0 = 2.d0*atan(sqrt((1-e)/(1+e))*tan(v0/2.d0))
      m0 = nula2pi(e0-e*sin(e0))
      omega0 = nula2pi(atan2(x(3)/sin(i0),x(1)*cos(omega)+
     &  x(2)*sin(omega))-v0)

      elmts(1)=a
      elmts(2)=e
      elmts(3)=i0
      elmts(4)=omega0	! node
      elmts(5)=omega	! peri
      elmts(6)=m0

      return
      end

c**********************************************************************

      real*8 function jdate2bessel(jdate)
      real*8 jdate
      real*8 year_Bessel
      parameter(year_Bessel = 365.242189d0)

      jdate2bessel = 2000.0 + (jdate - 2451544.53)/year_Bessel

      return
      end


